///////////////////////////////////////////////////////////////////////////
// GHSC.CPP -- Grafik HSC Player v1.05a
//             copyright (c) 1995 Akintunde Omitowoju
//
// Special thanks to Stacy David Doss for checking my OOP code,
// and W. Stephen Manuel for the explanation of some low-level stuff.
//
// HSCOBJ.OBJ  copyright (c) 1993, 1994 Neo Software Productions
//
// Start:  June 10, 1994
// End: Dec. ?? 1995
//
// Last update(s):
// December ??, 1994  -- Initial release GHSC v1.05
// February 16, 1995  -- code clean up, and addition comments v1.05a
// June 18, 1995      -- re-programmed equalizer bars...more efficient and
//                       smaller code.
// August 18, 1995    -- added more "struct"ure to the HSC C++ object,
//                       fixed note display and corrected text output
//////////////////////////////////////////////////////////////////////////
#include <iostream.h>
#include <iomanip.h>
#include <string.h>
#include <conio.h>
#include <stdlib.h>
#include <dos.h>
#include <alloc.h>
#include "adlib.h"
#include "hscops.h"
#include "fq.h"
#include "image.h"

#ifdef __cplusplus
    #define __CPPARGS ...
#else
    #define __CPPARGS
#endif

#define BYTE char
#define UWORD unsigned int

extern "C" {
           void  far pascal _hscplayer();
           } // End of extern "C" {

typedef struct {
  BYTE note[9];       // actual notes (1 byte per channel)
  BYTE instr[9];      // actual instruments
  BYTE equalizer[9];  // equalizer value (ffh-0fh)
  BYTE peak[9];       // equalizer peaks (ffh = peak)
  BYTE pattern;       // actual pattern
  BYTE track;         // actual track
  BYTE position;      // actual position
  BYTE play;          // play on/off (0/1)
} hsc_player;

hsc_player *song_info = new hsc_player;
//char *song_info = new char[40];
UWORD    TSeg = FP_SEG(song_info),
         TOfs = FP_OFF(song_info);

// far pointer to text screen
char far *txtscrn = (char far*)MK_FP(0xb800, 0);

//unsigned int TSeg, TOfs, FSeg, FOfs;

char note_tbl[12][3] = {"C-", "C#", "D-", "D#", "E-", "F-", "F#",
                        "G-", "G#", "A-", "A#", "B-"};
unsigned int Seg_Arr[9] = {664,824,984,1144,1304,1464,1624,1784,1944};
unsigned int equ_lit_bar[] = {0x1AF0};
unsigned int equ_dim_bar[] = {0x12f0};

unsigned int channel_lit_str[1][9] = { {0x1843, 0x1868, 0x1861,0x186e,0x186e,0x1865,
                             0x186c,0x1820,0x1831}};
unsigned int channel_dim_str[1][9] = { {0x1743, 0x1768, 0x1761,0x176e,0x176e,0x1765,
                             0x176c,0x1720,0x1731}};

int const INTR = 0x1C;
unsigned int const AdPort = 0x388;

void interrupt ( *oldint1c)(__CPPARGS);
void interrupt newint1c(__CPPARGS);
void start_bars(void);
void stop_bars(void);
void curs_off(void);
void curs_on(void);
void putscrn(void);
void far get_info(void);

int main ()
{
    f_queue list;       // FILER object (for file listing)
    char ch;
    HscOps *HscSng;     // Song object
    char i;

clrscr();

// check for the presence of the OPL2 chip (Adlib chip or compatible)
  if (!AdlibPresent(AdPort))
    {
     cout << "Sorry, you need an Adlib or compatible board";
     cout << " to run this program.\n";
     exit(0);
    }
  else
    { // cool, it's there...display info for a few seconds
     cout << "Adlib board found at port " << hex << AdPort << "h...\n";
     delay(1000);
    }

// initialize song_info array
   _fmemset(song_info, 0, 40);

curs_off();  // turn of the cursor
putscrn();   // show main screen

list.get_listing();       // get list of the files in current directory

song_info->play = 1;        // prime the song_info array (song is playing)
do {
 if (ch == '\x20')        // space bar pressed?
   {
     ch = ' ';           // yup, print out the file list
     list.print_list();
     if (strlen(list.get_name()) != 0)
       { // string length > 0, file name was selected
         if (HscSng && (song_info->play+1 == 0))
           { // if there's a song, and the song is playing
             stop_bars();  // stop the equalizers
               HscSng->stop_song();    // stop the song
                 delete HscSng;        // free memory song used
           }
                HscSng = new HscOps;   // new song, new memory
         HscSng->load_song(list.get_name());  // read in the song
         gotoxy(4,3);
         textbackground(1);
         cprintf("%-12s", list.get_name()); // print song's name
         // clear note status area
         for (i=0; i<9; ++i)
         {
           gotoxy(30, 5+i);
           printf("%s", "   ");
         }
         HscSng->play_song();                 // play the song!
         start_bars();                        // hook in equalizer bars
       }
    }
    do {
     get_info();                            // get misc. info about the song
    } while (!kbhit());                     // loop if no keypress

    ch = getch();     // grab keypress

} while (ch != '\x1b');  // if not ESC key, loop

// the following code below frees up song memory if being used
         if (HscSng && (song_info->play+1 == 0))
           {
             stop_bars();
               HscSng->stop_song();
                 delete HscSng;
           }
         else
           {
             if (HscSng)
               {
                delete HscSng;
               }
           }

    delete [] song_info;    // free song_info array

    curs_on();        // cursor back on
    textcolor(7);
    textbackground(0);

// the asm code ensures we're in text mode 03h
   asm {
               mov       ah,0x03
               xor       bx,bx
               int       0x10
   }

    clrscr();  // give the user a nice clear screen

    return(0);
}

void start_bars(void)
///////////////////////////////////
// void start_bars(...)
//
// Hooks equalizer routine onto timer interrup 1ch
{
  oldint1c = getvect(INTR);
  setvect(INTR,newint1c);
}

void stop_bars(void)
///////////////////////////////////
// void stop_bars(...)
//
// Removes equalizer routine onto timer interrup 1ch
{
  setvect(INTR,oldint1c);
}

void far get_info(void)
{
    int nt;

    asm {
    mov si, TSeg
    mov es,si
    mov si, TOfs
    xor ax,ax
    mov ah,0x1
    }
   _hscplayer();

    gotoxy(21,15);
    printf("%02X", song_info->pattern);
    gotoxy(21,16);
    printf("%02X", song_info->track);
    gotoxy(21,17);
    printf("%02X", song_info->position);

    // update note display
    for(nt=0; nt<9; ++nt)
    {
       gotoxy(30, 5+nt);

       if (song_info->peak[nt])  // get current note peak
       {  // new note played
          printf("%s%d", note_tbl[(song_info->note[nt] % 12)], // note
              (char)(song_info->note[nt] / 12));  // octave
        // light up the note!
        txtscrn[((4+nt)*160)+(29<<1)+1] = 0x1b;
        txtscrn[((4+nt)*160)+(30<<1)+1] = 0x1b;
        txtscrn[((4+nt)*160)+(31<<1)+1] = 0x1b;
       }
      else
      { // no new note -- dim old note
        /////////////////////////////////////////////////////////////////
        // origianal formula for access text fore and back ground colors
        // txtscrn[((y-1)*160)+2*(x-1)+1] = (1<<4)|11;
        //                                   ^     ^
        //                 background color -|     |- foreground color
        // Note:
        // Code below has been optimized (a litte);
        /////////////////////////////////////////////////////////////////
        txtscrn[((4+nt)*160)+(29<<1)+1] = 0x19;
        txtscrn[((4+nt)*160)+(30<<1)+1] = 0x19;
        txtscrn[((4+nt)*160)+(31<<1)+1] = 0x19;
      }
    }
}

void interrupt newint1c(__CPPARGS)
{
    BYTE j;

    asm {
      mov si, TSeg
      mov es,si
      mov si, TOfs
      xor ax,ax
      mov ah,0x7
    }
   _hscplayer();

  for (BYTE i=0; i<9; ++i)
  {
    // draw out equlizer bar
    for (j=-1; j<song_info->equalizer[i]; ++j)
    {
      if (j>6)  // next bar color
        equ_lit_bar[0] += 0x0400;
      if(j>11)  // next bar color
         equ_lit_bar[0] -=0x0200;

      memcpy((void *)MK_FP(0xB800, (Seg_Arr[i])+(j<<1)+2), equ_lit_bar, 2);
     equ_lit_bar[0] =0x1af0;  // reset values
    }

    for (j=song_info->equalizer[i];j<15; ++j)
    {
      // clear to background equalizer bars
      if (j>6)
        equ_dim_bar[0] += 0x0400;
      if(j>11)
         equ_dim_bar[0] -=0x0200;

      memcpy((void *)MK_FP(0xB800, (Seg_Arr[i])+(j<<1)+2), equ_dim_bar, 2);
     equ_dim_bar[0]=0x12F0;  // reset values
    }

    if (!(unsigned int) song_info->peak[i])
    {
      channel_lit_str[0][8] += i;
      movmem(channel_lit_str, (void *) MK_FP(0xB800, Seg_Arr[i]-22),18);
    }
   else
    {
     channel_dim_str[0][8] += i;
     movmem(channel_dim_str, (void *) MK_FP(0xB800, Seg_Arr[i]-22),18);
    }
   channel_lit_str[0][8] = 0x1831;
   channel_dim_str[0][8] = 0x1731;
}

 while ( !(inport(0x3da) & 8));
    oldint1c();

}

void curs_off(void)
{
   asm {
               mov       ah,0x03
               xor       bx,bx
               int       0x10
               or        ch,0x20
               mov       ah,0x01
               int       0x10
   }
}

void curs_on(void)
{

    asm {
               mov       ah,0x03
               xor       bx,bx
               int       0x10
               and       ch,0x1F
               mov       ah,0x01
               int       0x10
    }

}

void putscrn(void)
{
   void far *screen = (void far *)0xB8000000; // SET POINTER TO SCREEN
   memcpy(screen,IMAGEDATA,4000);             // MOVE VIDEO MEMORY TO SCREEN



}
